/*
 * Created on 05.04.2006 for PIROL
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2519 $
 *  $Date: 2006-10-11 11:53:07 +0200 (Mi, 11 Okt 2006) $
 *  $Id: BesselEllipsoid.java 2519 2006-10-11 09:53:07Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.utilities.projection.ellipsoid;


/**
 * Class that hold information about the bessel ellipsoid
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2006),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2519 $
 * 
 */
public class BesselEllipsoid extends AbstractEllipsoid {
    
    /*
    protected final static double E0_bessel = cBessel  / radToDeg * ( 1.0 - 3./4. * eq2Bessel + 45./64. * Math.pow(e2Bessel,4.) - 175./256. * Math.pow(e2Bessel,6.) + 11025./16384. * Math.pow(e2Bessel,8.) - 43659./65536. * Math.pow(e2Bessel,10.));
    protected final static double F2_bessel = radToDeg * ( 3./8. * eq2Bessel - 3./16. * Math.pow(e2Bessel,4.) + 213./2048. * Math.pow(e2Bessel,6.) );
    protected final static double F4_bessel = radToDeg * ( 21./256. * Math.pow(e2Bessel,4.) - 21./256. * Math.pow(e2Bessel,6.) );
    protected final static double F6_bessel = radToDeg * ( 151./6144. * Math.pow(e2Bessel,6.) );
    */
    
    protected double c = Double.NaN;
    
    protected double E0 = Double.NaN, E2 = Double.NaN, E4 = Double.NaN, E6 = Double.NaN;
    protected double F2 = Double.NaN;
    protected double F4 = Double.NaN;
    protected double F6 = Double.NaN;
    
//    protected static double E0 = 111120.619608d;
//    protected static double F2 = 0.143885358d;
//    protected static double F4 = 0.000210790d;
//    protected static double F6 = 0.000000423d;
    

    /*
    protected final static double aBessel = 6377397.15508d;
    protected final static double bBessel = 6356078.96290d;
    protected final static double cBessel = 6366742.52d;
    protected final static double eqBessel = (aBessel*aBessel-bBessel*bBessel)/(aBessel*aBessel);
    protected final static double eBessel = Math.sqrt(eqBessel);
    protected final static double eq2Bessel = (aBessel*aBessel-bBessel*bBessel)/(bBessel*bBessel);
    protected final static double e2Bessel = Math.sqrt(eq2Bessel);
    */
    
    /**
     *@inheritDoc
     */
    public final double getA() {
        //     6377397.155080
        return 6377397.15508d;
    }

    /**
     *@inheritDoc
     */
    public final double getB() {
        //     6356078.962900
        return 6356078.96290d;
    }
    
    /**
     *return <code>c=a^2/b</code>
     */
    public final double getC_Labonde() {
        return 6366742.52d;
    }
    
    /**
     *@inheritDoc
     */
    public int getEllipsoidId() {
        return AbstractEllipsoid.ELLIPSOID_ID_BESSEL;
    }

    
    public final double getParamE0() {
        if (Double.isNaN(E0)){
            E0 = getC()  / radToDeg * ( 1.0 - 3./4. * getESquare2() + 45./64. * Math.pow(getE2(),4.) - 175./256. * Math.pow(getE2(),6.) + 11025./16384. * Math.pow(getE2(),8.) - 43659./65536. * Math.pow(getE2(),10.));
        }
        return E0;
    }
    
    public final double getParamE2() {
        if (Double.isNaN(E2)){
            E2 = getC() * ( -3./8. * getESquare2() + 15./32. * Math.pow(getE2(),4.) - 525./1024. * Math.pow(getE2(),6.) + 2205./4096. * Math.pow(getE2(),8.));
        }
        return E2;
    }
    
    public final double getParamE4() {
        if (Double.isNaN(E4)){
            E4 = getC() * ( 15./256. * Math.pow(getE2(),4.) - 105./1024. * Math.pow(getE2(),6.) +  2205./16384. * Math.pow(getE2(),8.));
        }
        return E4;
    }
    
    public final double getParamE6() {
        if (Double.isNaN(E6)){
            E6 = getC() * ( -35./3072. * Math.pow(getE2(),6.) +   315./12288. * Math.pow(getE2(),8.));
        }
        return E6;
    }

    public final double getParamF2() {
        if (Double.isNaN(F2)){
            F2 = radToDeg * ( 3./8. * getESquare2() - 3./16. * Math.pow(getE2(),4.) + 213./2048. * Math.pow(getE2(),6.) - 255./4096. * Math.pow(getE2(),8.));
        }
        return F2;
    }

    public final double getParamF4() {
        if (Double.isNaN(F4)){
            F4 = radToDeg * ( 21./256. * Math.pow(getE2(),4.) - 21./256. * Math.pow(getE2(),6.) + 533./8192. * Math.pow(getE2(),8.));
        }
        return F4;
    }

    public final double getParamF6() {
        if (Double.isNaN(F6)){
            F6 = radToDeg * ( 151./6144. * Math.pow(getE2(),6.) -  453./12277. * Math.pow(getE2(),8.));
        }
        return F6;
    }

}
